/*
 * Author: Radovan Cerveny
 */

#ifndef MEASUREMENT_PROVISIONER_HPP_
#define MEASUREMENT_PROVISIONER_HPP_

#include "MeasurementProvisionerConfiguration.hpp"
#include "MeasurementProvisionerResults.hpp"
#include "MeasurementProvisionerInputData.hpp"
#include "MeasurementProvisionerUtils.hpp"
#include <fdstream>

#include <list>
#include <set>
#include <map>

namespace measurements {

class MeasurementProvisioner {
	static void prepareEnvironment ( const MeasurementProvisionerConfiguration & );
	static MPRPipelineResults runPipeline ( const MPPipeline &, const MPSubstitutionMap &, const MeasurementProvisionerConfiguration & );

	struct PipelineRunnerEnvironment {
		MPUtils::ShmFileHandle measurementsTmpfile, inputTmpfile, outputTmpfile, errorTmpfile;

		int measurementsFd, stdinFd, stdoutFd, stderrFd;

	public:
		PipelineRunnerEnvironment ( );
		~PipelineRunnerEnvironment ( );

		void commandFdInit ( );
		void commandFdSwap ( );
		void commandFdEnd ( );

		std::string		   retrievePipelineError ( );
		MeasurementResults retrieveMeasurementResults ( );
	};

public:
	static MeasurementProvisionerResults runConfiguration ( const MeasurementProvisionerConfiguration & );
};

}

#endif /* MEASUREMENT_PROVISIONER_HPP_ */
